"""Module containing the only function of the whole package: 'run_umodl'."""

import subprocess
import importlib.resources
import platform

umodl = importlib.resources.files("umodlwrapper") / ("umodl.exe" if platform.system() == "Windows" else "umodl")

def run_umodl(txtfile, kdicfile, kdictname, treatmentcolname, targetcolname):
    """Run the umodl executable.

    Parameters
    ----------
    txtfile: path-like
        Path to the Khiops .txt file containing the data.
    kdicfile: path-like
        Path to the Khiops .kdic file containing the Khiops Dictionary describing the data columns.
    kdictname: str
        The name of the Khiops Dictionary as found in kdicfile.
    treatmentcolname: str
        The name of the treatment column as found in txtfile and kdicfile.
    targetcolname: str
        The name of the target column as found in txtfile and kdicfile.

    Returns
    -------
    subprocess.CompletedProcess:
        The completed subprocess that has been run.

    Raises
    ------
    subprocess.CalledProcessError:
        If an error happened during the execution of the umodl executable.
    """
    return subprocess.run([umodl, txtfile, kdicfile, kdictname, treatmentcolname, targetcolname], check=True)



# Copyright (c) 2025 Orange. All rights reserved.
# This software is distributed under the BSD 3-Clause-clear License, the text of which is available
# at https://spdx.org/licenses/BSD-3-Clause-Clear.html or see the "LICENSE" file for more details.

"""Updates the copyright notice of the input files"""

import argparse
import filecmp
import os
import tempfile
import sys
import shutil
from datetime import datetime

# pylint: disable=line-too-long
byte_linesep = bytes(os.linesep, encoding="ascii")
copyright_banner_lines = [
    bytes(
        f"Copyright (c) {datetime.today().year} Orange. All rights reserved.",
        encoding="ascii",
    ),
    b"This software is distributed under the BSD 3-Clause-clear License, the text of which is available",
    b'at https://spdx.org/licenses/BSD-3-Clause-Clear.html or see the "LICENSE" file for more details.',
]
# pylint: enable=line-too-long


def main(args):
    """Main method"""
    # Process files and keep track if there were modifications
    were_files_modified = False
    for file_path in args.file_paths:
        is_file_modified = update_copyright(file_path)
        if is_file_modified:
            were_files_modified = True
            print(f"Updated {file_path}")

    # Set the return code
    return_code = 0
    if were_files_modified:
        return_code = 1

    return return_code


def update_copyright(file_path):
    """Updates the copyright notice of a file if necessary"""

    # Obtain the comment prefix string from the file extension
    _, ext = os.path.splitext(file_path)
    if ext in (".h", ".c", ".hpp", ".cpp", ".java"):
        comment_prefix = b"// "
    elif ext == ".py":
        comment_prefix = b"# "
    else:
        raise ValueError(f"Unsupported file extension '{ext}'.")

    # Read the lines from the source file
    with open(file_path, "rb") as file:
        lines = file.readlines()

    # Write the contents to a temporary file
    # NB: We clean manually the tmp file because the auto-delete creates problems in Win
    is_file_modified = False
    skipped_copyright = False
    with tempfile.NamedTemporaryFile(delete=False) as tmp_stream:
        # Write the current copyright, followed by an empty line
        for line in copyright_banner_lines:
            tmp_stream.write(comment_prefix)
            tmp_stream.write(line)
            tmp_stream.write(byte_linesep)
        tmp_stream.write(byte_linesep)

        # Rewrite the file as follows
        # - Skip the old copyright
        # - Write the rest of the lines, the last one without newline
        line_number = len(lines)
        for n, line in enumerate(lines):
            line = line.rstrip()
            if (
                line.startswith(comment_prefix + b"Copyright (c)")
                or line.startswith(comment_prefix + b"This software is distributed")
                or line.startswith(comment_prefix + b"at https://spdx.org")
            ) and not skipped_copyright:
                continue
            else:
                if not skipped_copyright:
                    # Skip head empty lines
                    if len(line) == 0:
                        continue
                    else:
                        skipped_copyright = True
                # Beware: all lines must end with an end of line, including the last line
                # (otherwise, the Windows RC compiler does not work)
                tmp_stream.write(line)
                tmp_stream.write(byte_linesep)

        # Flush the file contents
        tmp_stream.flush()

        # If the temporary file contents do not match the original, replace it
        tmp_file_path = tmp_stream.name
        if not filecmp.cmp(file_path, tmp_file_path, shallow=False):
            is_file_modified = True
            shutil.copyfile(tmp_stream.name, file_path)

    # Clean the temporary file
    os.remove(tmp_file_path)

    return is_file_modified


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="python update_copyright.py",
        formatter_class=argparse.RawTextHelpFormatter,
        description="Updates the copyright notice of the input files",
    )
    parser.add_argument(
        "file_paths",
        metavar="FILE",
        nargs="+",
        help="One or more source code files",
    )
    sys.exit(main(parser.parse_args()))



# Copyright (c) 2024 Orange. All rights reserved.
# This software is distributed under the BSD 3-Clause-clear License, the text of which is available
# at https://spdx.org/licenses/BSD-3-Clause-Clear.html or see the "LICENSE" file for more details.

import os
import string
import sys

# Files that must have non ASCII characters
ignored_file_paths = ["src/Learning/KWTest/KWTextParser.cpp"]

# Check the input files
file_paths = sys.argv[1:]
errors = False
for file_path in file_paths:
    # Ignore C++ files
    name, extension = os.path.splitext(file_path)
    if extension not in (".cpp", ".h"):
        sys.exit(0)

    # Ignore exceptions
    if file_path in ignored_file_paths:
        sys.exit(0)

    # Try to read the file as ASCII
    try:
        with open(file_path, encoding="ascii") as file:
            file.read()
    # On failure show the lines that contain characters out of range
    except UnicodeDecodeError:
        with open(file_path, encoding="utf8", errors="replace") as file:
            lines = file.readlines()
            for line_number, line in enumerate(lines, 1):
                if any(ord(char) > 127 for char in line):
                    print(f"{file_path}:{line_number}: {line}", end="")
        sys.exit(1)



# Copyright (c) 2024 Orange. All rights reserved.
# This software is distributed under the BSD 3-Clause-clear License, the text of which is available
# at https://spdx.org/licenses/BSD-3-Clause-Clear.html or see the "LICENSE" file for more details.

import sys
import os
from subprocess import Popen, PIPE


# script called from  generate_gui_add_view.cmake
# The argument is the command line to launch genere, for example:
# /bin//genere -outputdir /khiops/src/Learning/KWLearningProblem KWAnalysisSpec Parameters \
#  KWAnalysisSpec.dd KWAnalysisSpec.dd.log
# The command line is executed as it is and the stdout is redirect to the log file (last element of the command line).
# The content of the log is displayed to stdout
# If the command fails, the log file is removed and it exits with an error
def main():
    args = sys.argv[1:]
    log_path = args[-1]
    with open(log_path, "w") as log_file:
        process = Popen(args[:-1], stdout=log_file)
        process.wait()
        log_file.close()
        with open(log_path, "r") as log_file:
            for line in log_file:
                print("genere: " + line.strip())
        if process.returncode != 0:
            os.remove(log_path)
            exit(1)
        else:
            exit(0)


if __name__ == "__main__":
    main()



import setuptools
import setuptools.command.build
import subprocess
from pathlib import Path
from shutil import which, rmtree
import platform


class BuildC(setuptools.Command):
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        system = platform.system()
        if system == "Linux":
            preset = "linux-gcc-release"
        elif system == "Darwin":
            preset = "macos-clang-release"
        elif system == "Windows":
            preset = "windows-msvc-release"
        else:
            raise ValueError("Unsupported OS")
        pkgdirpath = next(Path(".").glob("build/lib.*"))  # Detect name of pkg directory since it is platform-dependant
        rmtree(pkgdirpath)
        pkgdirpath.mkdir()
        subprocess.run([which("cmake"),
                        "-B", "build/cmake",
                        "--preset", preset,
                        "-DMPI=OFF",
                        "-DTESTING=OFF"],
                        check=True)  # Generate build config files
        subprocess.run([which("cmake"), "--build", "build/cmake", "--target", "umodl"], check=True)  # Build
        for filename in ["README.md", "LICENSE"]:
            self.copy_file(filename, pkgdirpath / filename)
        (pkgdirpath / "umodlwrapper").mkdir()
        self.copy_tree("src/umodlwrapper", pkgdirpath / "umodlwrapper")
        if system == "Windows":
            self.copy_file("build/cmake/bin/umodl.exe", pkgdirpath / "umodlwrapper/umodl.exe")
        else:
            self.copy_file("build/cmake/bin/umodl", pkgdirpath / "umodlwrapper/umodl")


class Build(setuptools.command.build.build):
    sub_commands = setuptools.command.build.build.sub_commands + [('BuildC', None)]


class BinaryDistribution(setuptools.Distribution):
    def has_ext_modules(self):
        return True


setuptools.setup(
    cmdclass={
        'build': Build,
        'BuildC': BuildC
    },
    distclass=BinaryDistribution
)
